function [approx] = plateletApprox(obsData, varargin)

% Check the edge width of the 2^n x 2^n matrix
obsData_size = size(obsData);
if numel(obsData_size) < 1
  exception = MException('ArgumentError:Dimension', 'This is an image processing method; it cannot process a vector.');
  throw(exception);
elseif numel(obsData_size) > 2
  exception = MException('ArgumentError:Dimension', 'This is an image processing method; input matrix must be 2-dimensional');
  throw(exception);
end
edgeWidth = min(obsData_size);
if edgeWidth ~= max(obsData_size)
  exception = MException('ArgumentError:Dimension', 'Input matrix not square.');
  throw(exception);
end
if edgeWidth ~= 2^floor(log2(edgeWidth))
  exception = MException('ArgumentError:Dimension', 'Input matrix width not power of 2.');
  throw(exception);
end

% Set noiseType
if nargin < 2
  noiseType = 'Poisson';
else
  noiseType = varargin{1};
end
if strcmp(noiseType, 'Poisson')
  err = @(data, est) errPoisson(data, est);
elseif strcmp(noiseType, 'Gaussian')
  err = @(data, est) errGaussian(data, est);
else
  exception = MException('ArgumentError:Value', 'Noisetype must be either Poisson (default) or Gaussian.');
  throw(exception);
end

% Set penalty
if nargin < 4
  if strcmp(noiseType, 'Poisson')
    pen = log(sum(obsData(:)));
    pen_w = log(sum(obsData(:))) * 4/3;
  elseif strcmp(noiseType, 'Gaussian')
    pen = log(numel(obsData(:)));
    pen_w = log(numel(obsData(:))) * 4/3;
  else
    exception = MException('ArgumentError:Value', 'noiseType must be either Poisson or Gaussian.');
    throw(exception);
  end
else
  pen = varargin{2};
  if(pen <= 0)
    exception = MException('ArgumentError:Value', 'pen must be positive');
    throw(exception);
  end
  pen_w = varargin{3};
  if(pen_w <= 0)
    exception = MException('ArgumentError:Value', 'pen_w must be positive');
    throw(exception);
  end
end

% Set numWedges
if nargin < 5
  numWedges = floor(numel(obsData).^(1/3));
else
  numWedges = floor(sqrt(varargin{4}));
end

% Create the estimate
[m n] = size(obsData);
L = min(floor(log2(n)),log2(64));
merges = zeros([size(obsData), L]);

parfor iL = 1:L
  dyadLen = 2^iL;
  
  % Initialize values
  plateMerge = zeros(size(obsData));
  plateError = ones(size(obsData)).*bitmax;
  pMergePlate = -ones(size(obsData)).*bitmax;
  % Wedge estimate preallocation
  wedgePlateMerge = zeros(size(obsData));
  wedgePlateError = ones(size(obsData)).*bitmax;
  pMergeWedgePlate = -ones(size(obsData)).*bitmax;
  % Combined estimate preallocation
  merge = zeros(size(obsData));
  pMerge = -ones(size(obsData));
  
  % fitPlatelet
  for im = 1:dyadLen:m
    for in = 1:dyadLen:n
      imageData = obsData(im:im+dyadLen-1, in:in+dyadLen-1);
      plateMerge(im:im+dyadLen-1, in:in+dyadLen-1) = fitPlatelet(imageData);
      plateError(im:im+dyadLen-1, in:in+dyadLen-1) = sum(sum(err(imageData, plateMerge(im:im+dyadLen-1, in:in+dyadLen-1))));
      pMergePlate(im:im+dyadLen-1, in:in+dyadLen-1) = -plateError(im, in) - pen;
    end
  end
  % fitWedgePlate
  if (dyadLen > 2) && (dyadLen < 64)
    wedgePlateMerge = zeros(size(obsData));
    num_r = floor(numWedges/sqrt(m/dyadLen));
    for s=0:num_r-1
      for s2=0:num_r-1
        r = dyadLen/sqrt(2) * s/(num_r+1);
        th = 2*pi * s2 / (num_r + 1);
        mask = getMask(r, th, dyadLen);
        if (any(mask(:)) && any(~mask(:)))
          for im = 1:dyadLen:m
            for in = 1:dyadLen:n
              imageData = obsData(im:im+dyadLen-1, in:in+dyadLen-1);
              wedgePlateEst = fitMaskedPlatelet(imageData, mask);
              currWedgePlateError = sum(sum(err(imageData, wedgePlateEst)));
              if currWedgePlateError < wedgePlateError(im, in)
                wedgePlateError(im:im+dyadLen-1, in:in+dyadLen-1) = currWedgePlateError;
                wedgePlateMerge(im:im+dyadLen-1, in:in+dyadLen-1) = wedgePlateEst;
                pMergeWedgePlate(im:im+dyadLen-1, in:in+dyadLen-1) = -wedgePlateError(im:im+dyadLen-1, in:in+dyadLen-1) - 2*pen_w;
              end
            end
          end
        end
      end
    end
  end
  % Combine fitPlate and fitWedgePlate
  if (dyadLen > 2) && (dyadLen < 64)
    for im = 1:dyadLen:m
      for in = 1:dyadLen:n
        if pMergeWedgePlate(im, in) > pMergePlate(im, in)
          pMerge(im, in) = pMergeWedgePlate(im, in);
          merge(im:im+dyadLen-1, in:in+dyadLen-1) = wedgePlateMerge(im:im+dyadLen-1, in:in+dyadLen-1);
        else
          pMerge(im, in) = pMergePlate(im, in);
          merge(im:im+dyadLen-1, in:in+dyadLen-1) = plateMerge(im:im+dyadLen-1, in:in+dyadLen-1);
        end
      end
    end
  else
    merge = plateMerge;
    pMerge = pMergePlate;
  end
  merges(:,:,iL) = merge;
  pMerges(:,:,iL) = pMerge;
end
% Reassemble the merges
partition = ones(size(obsData));
y = obsData;
error = err(obsData, merges(:,:,1));
for iL=1:L
  dyadLen = 2^iL;
  merge = merges(:,:,iL);
  pMerge = pMerges(:,:,iL);
  for im = 1:dyadLen:m
    for in = 1:dyadLen:n
      pSplit = sum(sum(-error(im:im+dyadLen-1, in:in+dyadLen-1))) - sum(sum(partition(im:im+dyadLen-1, in:in+dyadLen-1)))*pen;
      if pMerge(im, in) >= pSplit
        y(im:im+dyadLen-1, in:in+dyadLen-1) = merge(im:im+dyadLen-1, in:in+dyadLen-1);
        if(iL < L)
          error(im:im+dyadLen-1, in:in+dyadLen-1) = err(obsData(im:im+dyadLen-1, in:in+dyadLen-1), y(im:im+dyadLen-1, in:in+dyadLen-1));
          partition(im:im+dyadLen-1, in:in+dyadLen-1) = 1/dyadLen^2;
        end
      end
    end
  end
end
approx = y;
